#include <cstdio>
#include <vector>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <queue>
#include <algorithm>
#include "hard_levels.h"
using namespace std;

int field[150][150];
int dists[150][150];

int DX[] = {0, 1, 0, -1};
int DY[] = {1, 0, -1, 0};
int DL[] = {'D', 'R', 'U', 'L'};
int DR[] = {'U', 'L', 'D', 'R'};

void fill_field(int index, int width, int height) {
  if (index == 0) for (int j = 0; j < height; j++) for (int i = 0; i < width; i++) field[j][i] = level0[j][i];
  if (index == 1) for (int j = 0; j < height; j++) for (int i = 0; i < width; i++) field[j][i] = level1[j][i];
  if (index == 2) for (int j = 0; j < height; j++) for (int i = 0; i < width; i++) field[j][i] = level2[j][i];
  if (index == 3) for (int j = 0; j < height; j++) for (int i = 0; i < width; i++) field[j][i] = level3[j][i];
  if (index == 4) for (int j = 0; j < height; j++) for (int i = 0; i < width; i++) field[j][i] = level4[j][i];
  if (index == 5) for (int j = 0; j < height; j++) for (int i = 0; i < width; i++) field[j][i] = level5[j][i];
  if (index == 6) for (int j = 0; j < height; j++) for (int i = 0; i < width; i++) field[j][i] = level6[j][i];
}

int width, height;

vector<vector<int> > formula;

void solve() {
  vector<int> values;
  
  int playerX = 0, playerY = 1;
  
  /* READ FORMULA */
  for (int j = 1; j < height - 1; j++)
    for (int i = 1; i < width - 1; i++) {      
      if (field[j][i] == 2) {
        playerX = i;
        playerY = j;
      }
      
      if (field[j][i] >= 100 && field[j][i] < 200)
        values.push_back(0);
      
      if (field[j][i] >= 200) {
        if (field[j][i - 1] >= 200 && field[j][i + 1] >= 200) {
          vector<int> clause;          
          clause.push_back(field[j][i]);          
          clause.push_back(field[j][i - 1]);                    
          clause.push_back(field[j][i + 1]);                              
          formula.push_back(clause);
        }
        
        if (field[j - 1][i] >= 200 && field[j + 1][i] >= 200) {
          vector<int> clause;          
          clause.push_back(field[j][i]);          
          clause.push_back(field[j - 1][i]);                    
          clause.push_back(field[j + 1][i]);                              
          formula.push_back(clause);
        }        
      }
    }
    
  for (int i = 0; i < (int)formula.size(); i++)
    for (int j = 0; j < 3; j++)
      if (formula[i][j] >= 200 && formula[i][j] < 300)
        formula[i][j] = formula[i][j] - 200 + 1;
      else
        formula[i][j] = -(formula[i][j] - 300 + 1);
  

        
  /* SCHONING'S ALGORITHM TO SOLVE SAT */
  int V = (int)values.size();
  int C = (int)formula.size();
      
  srand(time(NULL));
  
  bool satisfiable = false;
    
  long long int max_tries = pow(4.0 / 3.0, V) * V;

  for (long long int tries = 0; tries < max_tries; tries++) {
  
    for (int i = 0; i < V; i++)
      values[i] = (bool)(rand() % 2);
      
    for (int steps = 0; steps < 3 * V; steps++) {
      satisfiable = true;
      int wrong_clause = -1;
      
      for (int i = 0; i < C; i++) {
        bool clause_sat = false;

        for (int j = 0; j < 3; j++)
          if (values[abs(formula[i][j]) - 1] == (formula[i][j] > 0)) {
            clause_sat = true;
            break;
          }
              
        if (!clause_sat) {
          satisfiable = false;
          wrong_clause = i;
          break;
        }
        
      }
      
      if (satisfiable) break;
            
      int flipped_variable = abs(formula[wrong_clause][rand() % 3]) - 1;
      values[flipped_variable] = !values[flipped_variable];
    }
    
    if (satisfiable) break;
  }
  
  /* PATHFINDING */
  for (int m = 0; m <= (int)values.size(); m++)
    if (m == (int)values.size() || values[m]) {
      
      queue<pair<int, int> > Q;
      
      Q.push(make_pair(playerX, playerY));
      
      for (int j = 0; j < 150; j++)
        for (int i = 0; i < 150; i++)
          dists[j][i] = 123456789;

      dists[playerY][playerX] = 0;
      int cx = 0, cy = 0;
        
      while (!Q.empty()) {
        int x = Q.front().first;
        int y = Q.front().second;
        
        if ((m == (int)values.size() && field[y][x] == 3) || (m < (int)values.size() && field[y][x] == 100 + m)) {
          cx = x;
          cy = y;
          break;
        }
        
        Q.pop();
        
        for (int k = 0; k < 4; k++)
          if (field[y + DY[k]][x + DX[k]] != 0 && (field[y + DY[k]][x + DX[k]] < 200 || field[y + DY[k]][x + DX[k]] >= 300))
            if (dists[y + DY[k]][x + DX[k]] > dists[y][x] + 1) {
              dists[y + DY[k]][x + DX[k]] = dists[y][x] + 1;
              Q.push(make_pair(x + DX[k], y + DY[k]));
            }
            
      }
                              
      vector<int> v;
      int sx = cx;
      int sy = cy;
      
      while (sx != playerX || sy != playerY) {
        
        for (int k = 0; k < 4; k++)
          if (dists[sy + DY[k]][sx + DX[k]] == dists[sy][sx] - 1) {
            sx = sx + DX[k];
            sy = sy + DY[k];
            
            v.push_back(DR[k]);
            break;
          }
      }
            
      reverse(v.begin(), v.end());
      
      for (int j = 0; j < height; j++)
        for (int i = 0; i < width; i++)
          if (field[j][i] == 200 + m)
            field[j][i] = 300 + m;
          else if (field[j][i] == 300 + m)
            field[j][i] = 200 + m;
     
      for (int k = 0; k < (int)v.size(); k++)
        printf("%c", v[k]);
      printf("P");
      
      playerX = cx;
      playerY = cy;
    }
        
  puts("\n");
}

int main() {
  for (int i = 0; i < 7; i++) {
    formula.clear();
    fill_field(i, widths[i], heights[i]);
    width = widths[i];
    height = heights[i];
    solve();
  }
  
  return 0;
}
